
// (c) COPYRIGHT URI/MIT 1996
// Please read the full copyright statement in the file COPYRIGH.  
//
// Authors:
//      jhrg,jimg       James Gallagher (jgallagher@gso.uri.edu)

// Implementation for ClientArray. See ClientByte.cc
//
// jhrg 1/12/95

// $Log: ClientArray.cc,v $
// Revision 1.14.12.2  2000/06/14 20:42:07  jimg
// Added instrumentation in the dtor.
//
// Revision 1.14.12.1  2000/06/02 23:03:31  jimg
// Fixes for AttrTables with no entries.
// Added support for name translation and a DODS_ML_Real_Name attribute.
// Changed the `size' attribtue's name to DODS_ML_Size.
//
// Revision 1.14  1999/03/24 06:23:42  brent
// convert String.h to std lib <string>, convert to packages regex -- B^2
//
// Revision 1.13  1998/11/19 20:53:33  jimg
// Fixed the extra newline bug.
//
// Revision 1.12  1998/09/16 23:02:18  jimg
// Removed write_val() and replaced it with print_val() and print_all_vals()
//
// Revision 1.11  1998/08/03 16:39:43  jimg
// Fixed write_val mfunc so that outermost is no longer used
//
// Revision 1.10  1997/10/04 00:39:58  jimg
// Added changes to support the new deserialize() mfunc and write_val().
//
// Revision 1.9  1997/04/19 00:54:47  jimg
// Changed ouput stream to simplify reading (for loaddods).
//
// Revision 1.8  1997/04/14 22:37:15  jimg
// Separate the Array description information (name, type and num of dimensions)
// from the list of dimension sizes by a newline to simplify parsing.
//
// Revision 1.7  1997/02/06 20:43:31  jimg
// Fixed log messages
//
// Revision 1.6  1997/01/10 06:49:26  jimg
// Changed call to name_map::lookup() so that non-alphanumerics are mapped to
// underscore. 
//
// Revision 1.5  1996/11/23 05:12:02  jimg
// Added support for variable renaming via the name_map object.
//
// Revision 1.4  1996/10/25 23:12:25  jimg
// Changed/Fixed write_val() so that it uses smart_newline().
//
// Revision 1.3  1996/10/23 23:46:34  jimg
// Modified so that writeval outputs a `recursive' data stream. Thus any
// data type can be written and later read without being flattened. This
// change was made so that the NSCAT data which is represented as a Sequence
// of Structures.
//
// Revision 1.2  1996/10/07 21:03:14  jimg
// Fixed Header.
//
// Revision 1.1  1996/09/30 23:58:50  jimg
// Added.

#ifdef __GNUG__
#pragma implementation
#endif

#include "config_dap.h"

#include <assert.h>

#include <iostream.h>

#include <Pix.h>
#include <SLList.h>
#include <string>

#include "ClientArray.h"
#include "name_map.h"

extern name_map names;
extern bool translate;

#ifdef TRACE_NEW
#include "trace_new.h"
#endif

extern void smart_newline(ostream &os, Type type);

Array *
NewArray(const string &n, BaseType *v)
{
    return new ClientArray(n, v);
}

BaseType *
ClientArray::ptr_duplicate()
{
    return new ClientArray(*this);
}

ClientArray::ClientArray(const string &n, BaseType *v) : Array(n, v)
{
}

ClientArray::~ClientArray()
{
    DBG(cerr << "Entering ~ClientArray (" << this << ")" << endl);
    DBG(cerr << "Exiting ~ClientArray" << endl);
}

bool
ClientArray::read(const string &, int &)
{
    assert(false);
    return false;
}

void 
ClientArray::print_val(ostream &os, string, bool print_decl_p)
{
    if (print_decl_p) {
	os << type_name() << endl << var()->type_name() << " " 
	   << names.lookup(name(), translate) << " " << dimensions(true)
	   << endl;

	// Write the actual dimension sizes on a spearate line.
	for (Pix p = first_dim(); p; next_dim(p))
	    os << dimension_size(p, true) << " ";

	os << endl;
    }

    for (int i = 0; i < length(); ++i)
	var(i)->print_val(os, "", false);
}    

AttrTable &
ClientArray::getAttrTable()
{
    return _attr;
}

void 
ClientArray::setAttrTable(AttrTable &attr)
{
    _attr = attr;
}
